%%%%%%%%%%%%%%%%%%%%%% ep10_2.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB 
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
%      Exploration Problem 10.2 
%	does observer-controller design for 3rd order
%	system used in all examples plant with 2 real 
%	poles, no zeros, sensor with 1 time constant
%----------------------------------------------------
clear;clf
% 
disp('EP10.2: observer-controller design for')
disp('.....plant with 2 real poles, no zeros')
disp('.....sensor with 1 time constant')
%
tauP1 = 2; tauP2 = 0.5; Kproc = 4; % plant
tau_sen = 0.05;      % sensor
numGp = Kproc;
denGp = conv([tauP1 1],[tauP2 1]);
Gp = tf(numGp,denGp);
disp('SS model of plant')
Gp_ss = ss(Gp)
[Ap,Bp,Cp,Dp] = ssdata(Gp_ss);
disp('*******>');pause
numH = 1;
denH = [tau_sen 1];
H = tf(numH,denH);
disp('SS model of sensor')
Hss = ss(H)
[Asen,Bsen,Csen,Dsen] = ssdata(Hss);
disp('*******>');pause
disp('SS model of plant & sensor in series')
Gps = Hss*Gp_ss;
[Aps,Bps,Cps,Dps] = ssdata(Gps)
disp('*******>');pause
%----- check for controllability & observability
disp('both of these rank values should be 3')
n_ctrb = rank(ctrb(Aps,Bps))
n_obsv = rank(obsv(Aps,Cps))
disp('*******>');pause
%
disp('Reasonable desired CL eigenvalues are: [-1; -2; -20]')
pp = input('enter 3 desired CL eigenvalues as column.....')
Rpp_str = vec2str(real(pp));
Ipp_str = vec2str(imag(pp));
disp('Controller gains')
K = place(Aps,Bps,pp)
disp('*******>');pause
disp('Gains for observer poles = 2 x CL eigenvalues')
L = place(Aps',Cps',2*pp)'
disp('*******>');pause
disp('A matrix of observer-controller')
Aoc = Aps-Bps*K-L*Cps	% observer-controller (3 x 3)
disp('*******>');pause
disp('Input matrix of observer-controller')
Boc = L			%     (3x1)
disp('*******>');pause
disp('Output matrix of observer-controller')
Coc = K		%     (1x3)
Doc = zeros(1,1)	%     (1x1)
disp('*******>');pause
disp('Observer-controller poles')
eig_OC = eig(Aoc)		% obsvr-ctrlr poles
disp('*******>');pause
disp('Observer-controller zeros')
Goc_ss = ss(Aoc,Boc,Coc,Doc);
zero_OC = tzero(Goc_ss)	% obsv-ctrlr zeros
disp('*******>');pause
disp('Modify plant+sensor model so plant output is ')
disp('...... 2nd output so we can plot it rather') 
disp('.......than sensor output')
%Cps2 = [Cps; [Cp 0]]
Cps2 = [Cps; [0 Cp]]
Dps2 = [Dps; Dp]
Gps2_ss = ss(Aps,Bps,Cps2,Dps2);
disp('*******>');pause
%
disp('Series connection of obsvr-ctrlr & plant+sensor')
disp('..... having 1 input & 2 outputs')
%Gocp = Goc_ss*Gps2_ss;
Gocp = Gps2_ss*Goc_ss;
[Aocp,Bocp,Cocp,Docp] = ssdata(Gocp);
Aoc
disp('*******>');pause
Boc
disp('*******>');pause
Coc, Doc
disp('*******>');pause
disp('Unity fdbk connection using only sensor output')
disp('.....positive sign at summing junction')
outputs = 1;
inputs = 1;
Gcl = feedback(Gocp,1,inputs,outputs);
[Acl,Bcl,Ccl,Dcl] = ssdata(Gcl);
Acl
disp('*******>');pause
Bcl
disp('*******>');pause
Ccl, Dcl
disp('*******>');pause
disp('closed-loop eigenvalues are:')
ev_CL = eig(Acl)
disp('*******>');pause
disp('prefilter to correct DC gain')
kk = dcgain(Gcl);	% 2x1 vector
K1 = 1/kk(2)
disp('*******>');pause
disp('Put gain in front of CL system')
Gcl2 = Gcl*K1;
[Acl2,Bcl2,Ccl2,Dcl2] = ssdata(Gcl2);
Acl2
disp('*******>');pause
Bcl2
disp('*******>');pause
Ccl2, Dcl2
disp('*******>');pause
disp('Unit step in reference input')
t = [0:0.02:4]';
ref = ones(size(t));
[y,t,x] = lsim(Gcl2,ref,t);
plot(t,y(:,2),t,y(:,1),'--');grid
title('EP10.2: 2nd-order plant (solid) + sensor (dashed)')
xlabel('time (secs)')
ylabel('Amplitude')
text(1,0.25,'Controller poles')
text(1,0.20,['real parts = ' Rpp_str])
text(1,0.15,['imag parts = ' Ipp_str])
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
