%%%%%%%%%%%%%%%%%%%%%% ch6ex.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Dec. 98
%----------------------------------------------------
%  This MATLAB file runs the 4 examples that appear 
%  inChapter 6 (Frequency Response Analysis) in the 
%  BookWare Companion Feedback Control Problems Using 
%  MATLAB by D. K. Frederick and J. H. Chow 
%       *************************************
%       *   requires the special functions: *
%       *   cpole2t.m, rpole2t.m 
%       *************************************
%----------------------------------------------------
clear,clf
%  menu to select Example to be run
example = -1;      % force menu to appear
while (example < 1) | (example > 4),
  disp(' 1 <= Example 6.1: Steady-state sinusoidal response')
  disp(' 2 <= Example 6.2: Bode, Nichols and Nyquist plots')
  disp(' 3 <= Example 6.3: Break frequencies and asymptotes')
  disp(' 4 <= Example 6.4: Crossover frequencies, margins & stability')
  example = input('Select an example, or enter 0 to return to chapter menu:  ');

if example == 0, disp('Completed Chapter 6 examples'), return, end

if example == 1,
  disp('Example 6.1: Steady-state sinusoidal response')
  %   
  disp(' ')
  disp('Create G(s) as a TF object')
  G = tf([ 10 50 ],[ 1 4 3 ])
  disp('*****>');pause
  disp('Setting up t, u and y')
  t = [0:0.06:6]'; % time vector with 0.06 sec. increment
  u = 2*cos(5*t + 30*pi/180);  
  y = lsim(G,u,t);
  disp('magnitude and phase of frequency response at w = 5 rad/s')
  [mag,phase] = bode(G,5)  
  disp('*****>');pause
  disp('magnitude and phase of steady-state response')
  y_mag = 2 * mag
  y_phase = 30 + phase
  y_ss = 2*mag*cos (5*t + (30+phase)*pi/180);
  plot(t,u,'-',t,y,'--',t,y_ss,'-.')
  grid, title('Ex 6.1 : u (solid), y (dashed) & yss (dashdot)')
  xlabel('Time (secs)')
  ylabel('u, y, yss')
  text(0.7,2,'u(t)')
  text(1.1,-6,'y(t)')
  text(0.4,4,'yss(t)')
  keyboard
  clear G mag phase y_mag y_phase t u y y_ss
  disp('end of Example 6.1'), disp('  ')
  example =-1;     % force menu to appear
end     % of example 6.1 so return to menu
%-------------------------------------

if example == 2,
  disp('Example 6.2: Bode, Nichols and Nyquist plots')
  disp(' ')
  disp('Create G(s) as a TF object')
  numG = [ 1280 640 ]
  denG = [ 1 24.2 1604.81 320.24 16 ]
  G = tf(numG,denG)
  disp('*****>');pause
  disp('Make Bode plot directly from the bode command')
  w = logspace(-2,3,100)';  
  bode(G,w);
  %subplot(2,1,1), title('Ex 6.2 : Bode plot ')
  keyboard
  disp('Make Nichols plot directly from the nichols command')
  disp('  The encirclement direction is indicated')
  %subplot(1,1,1), 
  nichols(G,w), grid, ngrid
  %title('Ex 6.2 : Nichols plot')
  %hold on
  %plot([-180 -180],[-400 400])  % make solid line for axis
  %plot([-1000 1000],[0 0])
  %hold off
  axis([-270 0 -40 40])
  keyboard
  disp('Make Nyquist plot directly from the nyquist command')
  nyquist(G);
  grid, %title('Ex 6.2 : Nyquist plot')
  axis([-10 50 -30 30])
  axis equal
  keyboard
  disp('Another Nyquist plot showing the detail about the -1 point.')
  disp('  We will put a unit circle about the origin on the plot.')
  hold on
  %  make unit circle
  w = [0:2*pi/100:2*pi];
  ucir = exp(j*w);
  ucir_re = real(ucir); ucir_im = imag(ucir);
  plot(ucir_re,ucir_im);
  hold off
  axis([-1 1 -1 1])
%  axis equal 
  keyboard
  axis('normal')
  axis('auto')
  clear numG denG G w fr_re fr_im ucir ucir_re c ucir_im 
  disp('end of Example 6.2'), disp('  ')
  example =-1;     % force menu to appear
end     % of Example 4.2 so return to menu
%-------------------------------------
if example == 3,
  disp('Example 6.3: Break frequencies and asymptotes')
  disp(' ')
  numG = [ 1280 640 ]
  denG = [ 1 24.2 1604.81 320.24 16 ]
  G = tf(numG,denG)
  disp('*****>');pause
  disp('compute system poles and zeros')
  [z,p,k] = zpkdata(G,'v');
  disp('zeros')
  damp(z)
  disp('poles')
  damp(p)
  disp('*****>');pause
  disp('Breakfrequencies due to zeros, rad/s')
  abs(z)
  disp('Breakfrequencies due to poles, rad/s')
  abs(p)
  disp('*****>');pause
  disp('Overlay magnitude plot with asymptotes')
  w = logspace(-2,3,100)';  
  [mag,phase] = bode(G,w);
  mag = reshape(mag,[100 1]);  % reshape a 1x1x100 matrix into a 
  										 % 100x1 matrix
  semilogx(w,20*log10(mag),'--')  
  hold on
  lfg = dcgain(G);
  cf1_dB = 20*log10(lfg);
  plot([0.01; 0.1],[cf1_dB; cf1_dB],'-')
  cf2_dB = cf1_dB - 40*log10(0.5/0.1);
  plot([0.1; 0.5],[cf1_dB; cf2_dB],'-')
  cf3_dB = cf2_dB - 20*log10(40/0.5);
  plot([0.5; 40],[cf2_dB; cf3_dB],'-')
  end_dB = cf3_dB - 60*log10(1000/40);
  plot([40;1000],[cf3_dB; end_dB],'-')
  hold off 
  %labels for asymptotes
  text(0.23,25,'-40 dB/dec')
  text(1.5,-25,'-20 dB/dec')
  text(15,-70,'-60 dB/dec')
  axis([0.01 1000 -100 40])
  xlabel('Frequency (rad/sec)')
  ylabel('Gain dB')
  title('Ex 6.3 : Bode magnitude plot')
  grid
  keyboard
  %clear numG denG z p k 
  disp('end of Example 6.3'); disp('  ')
  example =-1;     % force menu to appear
end     % of Example 6.3 so return to menu
%-------------------------------------

if example == 4,
  disp('Example 6.4: Crossover frequencies, margins & stability')
  disp(' ')
  disp('Create G(s) as TF object')
  numG = [ 1280 640 ]
  denG = [ 1 24.2 1604.81 320.24 16]
  G = tf(numG,denG)
  disp('*****>');pause
  disp('Gain and phase margins and crossover frequencies')
  margin(G);
  %grid, subplot(2,1,1), grid
  disp(' ')
  disp('To find the gain and phase margins graphically, redo ')
  disp('  Example 6.2 and examine the Bode, Nichols and')
  disp('  Nyquist plots. Verify using the answers obtained here.')
  keyboard
  disp('Compute closed-loop system poles')
  G_CL = feedback(G,1)
  pole_CL = pole(G_CL)
  disp('*****>');pause
  clear numG denG G G_CL pole_CL
  disp('end of Example 6.4'); disp('  ')
  example =-1;     % force menu to appear
end     % of Example 6.4 so return to menu
%-------------------------------------

end             % of while
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%