%%%%%%%%%%%%%%%%%%%%%% ch7ex.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Nov. 94
%----------------------------------------------------
%  This MATLAB file runs the 3 examples that appear 
%  in Chapter 7 (System Performance) in the BookWare 
%  Companion Feedback Control Problems Using MATLAB
%  by D. K. Frederick and J. H. Chow 
%----------------------------------------------------
clear,clf
%  menu to select Example to be run
example = -1;      % force menu to appear
while (example < 0) | (example > 5),
  disp(' 1 <= Example 7.1: Step response specifications')
  disp(' 2 <= Example 7.2: Steady-state regulation')
  disp(' 3 <= Example 7.3: Bandwidth, gain & phase margins')
  example = input('Select an example, or enter 0 to quit ==> ');

if example == 0, disp('Completed Chapter 7 examples'), return, end

if example == 1,
  disp(' ')
  disp('Example 7.1: Step response specifications')
  disp(' ')
  format short e 
  denGp = conv(conv([1 0.01],[1 1]),[1 20])
  Gp = tf(1,denGp)
  G  = 50*Gp   % Gc(s) = 50
  disp('*****>');pause
  T = feedback(G,1.0)  % unity feedback
  t = [0:0.05:20]';
  yr = step(T,t);
  clf, plot(t,yr); grid
  ylabel('Amplitude');xlabel('Time (secs)')
  title('Ex 7.1: Step response of third order system')
  [Mo,tp,tr,ts,ess] = tstats(t,yr,1);
  text(8,0.42,['overshoot = ' num2str(Mo) ' %'])
  text(8,0.34,['time to peak = ' num2str(tp) ' s'])
  text(8,0.26,['rise time = ' num2str(tr) ' s'])
  text(8,0.18,['2% settling time = ' num2str(ts) ' s'])
  text(8,0.10,['steady-state error = ' num2str(ess) ' %'])
  damp(T)
  [numT,denT] = tfdata(T,'v')
  [resS,polS,otherS] = residue(numT,[denT 0])
  keyboard
  Td = feedback(Gp,50)
  step(Td,t)
  plt_ttl('Step Response',' ')
  grid
  title('Ex 7.1: Disturbance response of third order system')
  keyboard
  disp('end of Example 7.1')
  example =-1;     % force menu to appear
end     % of example 7.1 so return to menu
%-------------------------------------

if example == 2,
  disp(' ')
  disp('Example 7.2: Steady-state regulation')
  denGp = conv(conv([1 0],[1 1]),[1 20])     
  Gp = tf(1,denGp)                          % generate Gp(s)
  G = 50*Gp                                 % seires connection
  Kv = vgain(G)                             % velocity error constant
  disp('*****>');pause
  T = feedback(G,1)                         % unity feedback system
  t = [0:0.05:10]'; u = t;                  % time and unit ramp
  y = lsim(T,u,t);                          % ramp response
  plot(t,t,'-',t,y,'--')
  title('Ex 7.2: Ramp response')
  xlabel('Time (secs)')
  ylabel('Amplitude')
  text(4.3,5.5,'input')
  text(6.5,5.5,'response')
  grid
  keyboard
  disp('end of Example 7.2')
  example =-1;     % force menu to appear
end     % of Example 7.2 so return to menu
%-------------------------------------

if example == 3,
  disp(' ')
  disp('Example 7.3: Bandwidth, gain & phase margins')
  disp(' ')
  denGp = conv(conv([1 0.01],[1 1]),[1 20])  % generate Gp(s)
  Gp = tf(1,denGp)
  G  = 50*Gp				     % series connection 
  disp('*****>');pause
  margin(G)                    % gain & phase margins
  plt_ttl('Bode Diagrams',' ')
  keyboard
  Tr = feedback(G,1)           % CL transfer function T(s)
  bode(Tr)
  plt_ttl('Bode Diagrams',' ')
  title('Ex 7.3: CL transfer function frequency response')
  keyboard
  [mag_CL,ph_CL,w] = bode(Tr);             % frequency response
  mag_CLdB = 20*log10(mag_CL);             % gain in dB
  lfg_CL = dcgain(Tr)                      % dc gain
  lfg_CL = 20*log10(lfg_CL);
  BW = bwcalc(mag_CLdB,w,lfg_CL)           % bandwidth
  [M_pw,i] = max(mag_CLdB)                 % maximum freq. resp.
  wp = w(i)                                
  disp('*****>');pause
  S = feedback(1,G) 			   % S(s)
  [magS,phS] = bode(S,w);                  % sensitivity
  magS_dB = 20*log10(magS);
  [maxS,j] = max(magS_dB);
  w(j);
  subplot(1,1,1)
  semilogx(w,mag_CLdB(:),'-',w,magS_dB(:),'--'), grid
  title('Ex 7.3: Magnitude plots of T(jw) and S(jw)')
  xlabel('Frequency (rad/sec)')
  ylabel('Gain dB')
  text(12,5,'Sensitivity')
  text(12,-4,'peak = 6.45 dB')
  text(12,-9,'  at 1.80 rad/s')
  text(1.2,-43,'Magnitude response')
  text(1.2,-53,'CL bandwidth = 2.32 rad/s')
  text(1.2,-58,'Mp = 5.37 dB')
  text(1.2,-63,'wp = 1.52 rad/s') 
  keyboard
  disp('end of Example 7.3')
  example =-1;     % force menu to appear
end     % of Example 7.3 so return to menu
%-------------------------------------

end             % of while
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
