%%%%%%%%%%%%%%%%%%%%%% cp7_1.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/9/99 HS
%----------------------------------------------------
% .M file for analyzing the power system example for 
%            Comprehensive Problem CP7.1
%----------------------------------------------------
clear,clf
disp('Comprehensive Problem CP7.1')

epow    % read data

disp('Power system transfer function, with V_term as output')
num = num1
den
disp('*****>'), pause
disp('lag controller with alpha = 10 and selected Klag and zlag')
Klag = [10 20 10]
zlag = [-1 -1 -2]
disp('*****>');pause
for i = 1:3
  K_lag = Klag(i) 
  K_lag_str = num2str(Klag(i));
  Z_lag=zlag(i)
  zlag_str = num2str(zlag(i));
  alpha = 10;
  alpha_str = num2str(alpha);
  disp('*****>');pause
  numGc = Klag(i)*[1 -zlag(i)]
  denGc = [1 -zlag(i)/alpha]
  Gc = tf(numGc,denGc)
  numGp = num;
  denGp = den;
  Gp= tf(numGp,denGp)
  G = Gc*Gp
  disp('*****>');pause
  Tr = feedback(G,1)
  t = [0:0.05:20]';
  y = step(Tr,t);
  clf, plot(t,y); grid
  ylabel('Amplitude');xlabel('Time (sec)')
  title('CP7.1: Step response of power system')
  text(13,0.20,['Klag = ' K_lag_str])
  text(13,0.15,['zlag = ' zlag_str])
  text(13,0.10,['alpha = ' alpha_str])
  [Mo,tp,tr,ts,ess] = tstats(t,y,1.0);
  text(2,0.30,['overshoot = ' num2str(Mo) ' %'],'sc')
  text(2,0.25,['time to overshoot = ' num2str(tp) ' s'],'sc')
  text(2,0.20,['rise time = ' num2str(tr) ' s'],'sc')
  text(2,0.15,['2pct settling time = ' num2str(ts) ' s'],'sc')
  text(2,0.10,['steady-state error = ' num2str(ess) ' %'],'sc')
  disp('damping ratios and natural frequencies')
  damp(Tr)
  keyboard
  disp('gain and phase margins')
  margin(G)                    			  % gain & phase margins
  keyboard
  bode(Tr)
  title('CP7.1: Frequency response of closed-loop transfer function')
  keyboard
  [magCL,phCL,w] = bode(Tr);  		  	  % frequency response
  magCLdB = 20*log10(magCL);             % gain in dB
  lfgCL = dcgain(Tr)            % dc gain
  lfgCL_dB = 20*log10(lfgCL);
  BW = bwcalc(magCLdB,w,lfgCL)           % bandwidth
  [Mp,i] = max(magCLdB)                  % maximum freq. resp.
  wr = w(i)                                
  disp('*****>');pause
  S = feedback(1,G) 
  bode(S)
  title('CP7.1: Frequency response of closed-loop sensitivity function')
  keyboard
  [magS,phS] = bode(S,w);                % sensitivity
  magS_dB = 20*log10(magS);
  [maxS,imaxS] = max(magS_dB)
  w_maxS = w(imaxS)
  clf
  semilogx(w,magCLdB(:),'-',w,magS_dB(:),'--'), grid
  xlabel('Frequency (rad/sec)')
  ylabel('Gain dB')
  title('CP7.1: Sensitivity and transfer function response')
  text(20,5,'|S(jw)|')
  text(20,-40,'|T(jw)|') 
  text(1,-65,['Klag = ' K_lag_str])
  text(1,-70,['zlag = ' zlag_str])
  text(1,-75,['alpha = ' alpha_str])
  keyboard 
end
clear
disp('end of Comprehensive Example 7.1')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




