%%%%%%%%%%%%%%%% fig8_2c.m %%%%%%%%%%%%%%%
%
% 	draws figure 8.2(c)
%      2nd order plant with 2 real poles with P-only control
% 	runs reference & disturbance step response for KP 
%  	Jan 99
%   last revision:  1/6/99  JHC
%----------------------------------------
clear
set(gcf,'DefaultAxesFontSize',16)
set(gcf,'DefaultTextFontSize',16)
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
denGp = conv([tauP1 1],[tauP2 1]);
Gp    = tf(Kproc,denGp)
H     = tf(1,[tau_sen 1])
%---------- open-loop transfer function
GpH   = Gp*H;
KP = 1.255;
KP_str=num2str(KP);

%---------- open-loop transfer function for reference input
%----- closed-loop step response to reference input
T_ref = feedback(KP*Gp,H,-1)
t = [0:0.02:10];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
T_dist = feedback(Gp,-KP*H,+1)
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
%title('fig 8.2(c): CL ref & dist step responses, P-only control')
text(4.0,0.25,['KP = ' KP_str])
text(6.5,0.88,'reference')
text(6.5,0.71,'disturbance')
axis([0 10 0 1.2])
print -deps fig8_2c.eps
%-- reset fonts to defaults
set(gcf,'DefaultAxesFontSize',get(0,'DefaultAxesFontSize'));
set(gcf,'DefaultTextFontSize',get(0,'DefaultTextFontSize'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%