% Model file hydro.m
% This M-file contains the model of the hydro-turbine 
% system described in Appendix A of "Feedback 
% Control Problems using MATLAB and the Control System 
% Toolbox" by D. K. Frederick and J. H. Chow. The turbine 
% has 1 state, 1 input, and 1 output. The actuator has 
% 1 state, 1 input, and 1 output.

%%%%%%%%%%%%%%%%%%%%%% hydro.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB 
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%----------------------------------------------------

%  model parameters
Tw = 2;         % s, water time constant
TQ = 0.5;       % s, actuator time constant

%  transfer function
hydro_num = [-Tw 1];
hydro_den = [Tw/2 1];
[Ah,Bh,Ch,Dh] = tf2ss(hydro_num,hydro_den);

% actuator transfer function
act_num = 1;
act_den = [TQ 1];
[Aa,Ba,Ca,Da] = tf2ss(act_num,act_den);

%  total transfer function 
[numG,denG] = series(act_num,act_den,hydro_num,hydro_den);

[A,B,C,D] = series(Aa,Ba,Ca,Da,Ah,Bh,Ch,Dh);

save hydro.mat A B C D numG denG

disp('hydro-turbine system transfer function is in numG,denG')
disp('                     state-space model is in A B C D')
disp('System data is save in hydro.mat')
disp(' ')

%%%%%%%%%%%%%%%%%% end of hydro.m %%%%%%%%%%%%%%%%%%%
